﻿using System.Net.Http.Headers;
using System.Text;
using System.Xml.Linq;
using Newtonsoft.Json;

namespace NetBootServerCore.Components.DDNS.Providers;

public class CloudFlareDomainProvider : DomainProvider
{
    public override string GetIp(DomainConfigEntry config)
    {
        var req = new HttpRequestMessage(HttpMethod.Get,
            $"https://api.cloudflare.com/client/v4/zones/{config.Domain}/dns_records/{config.Key}");
        req.Headers.Authorization = new AuthenticationHeaderValue("Bearer", config.Secret);
        using var http = CreateHttpClient(config);
        var response = http.Send(req);
        response.EnsureSuccessStatusCode();

        var json = response.Content.ReadAsStringAsync().Result;
        var r = JsonConvert.DeserializeAnonymousType(json, new { result = new { content = string.Empty } });
        return r.result.content;
    }

    public override void PutIp(DomainConfigEntry config, string ip)
    {
        var req = new HttpRequestMessage(HttpMethod.Put, $"https://api.cloudflare.com/client/v4/zones/{config.Domain}/dns_records/{config.Key}");
        req.Headers.Authorization = new AuthenticationHeaderValue("Bearer", config.Secret);
        req.Content = JsonContent.Create(new { name = config.RecordName, content = ip, type = "A" });

        using var http = CreateHttpClient(config);
        var response = http.Send(req);
        response.EnsureSuccessStatusCode();
        var result = response.Content.ReadAsStringAsync().Result;
    }
}