﻿using Microsoft.AspNetCore.Http.Extensions;
using NetBootServerCore.Components.Logging;
using System.Collections.Immutable;
using LogLevel = Microsoft.Extensions.Logging.LogLevel;

namespace NetBootServerCore.Components.Http
{
    public class HttpServer : NbsComponentBase
    {
        private readonly string[] _args;
        private readonly IReadOnlyCollection<IHttpRequestProcessor> _processors;
        private WebApplication _app;

        public HttpServer(string[] args, IEnumerable<IHttpRequestProcessor> processors) : base(new Logger("HTTP"))
        {
            _args = args;
            _processors = processors.ToImmutableArray();
        }

        protected override void OnStart()
        {
            var builder = WebApplication.CreateBuilder(_args);
            builder.Logging.SetMinimumLevel(LogLevel.None);
            _app = builder.Build();

            _app.Run(async context =>
            {
                Logger.LogVerbose($"{context.Request.Method} {context.Request.Path}");

                context.Response.Headers.Server = nameof(NetBootServerCore);

                foreach (var processor in _processors)
                {
                    bool handled;
                    try
                    {
                        handled = await processor.ProcessRequest(context);
                    }
                    catch (Exception e)
                    {
                        Logger.LogError("ProcessRequest", e);
                        context.Response.StatusCode = 500;
                        return;
                    }
                    if (handled)
                    {
                        await context.Response.CompleteAsync();
                        return;
                    }
                }

                context.Response.StatusCode = 404;
                Logger.LogWarn($"Not handled request {context.Request.Method} {context.Request.GetEncodedPathAndQuery()}");
            });

            _app.RunAsync();
        }

        protected override void OnStop()
        {
            _app.StopAsync();
        }
    }
}
