﻿using NetBootServerCore.Components.Logging;
using NetBootServerCore.Components.NetBoot.Dhcp;
using Newtonsoft.Json;

namespace NetBootServerCore.Components.Http.Processors.WebApi;

public class WebApiProcessor : NbsComponentBase, IHttpRequestProcessor
{
    private readonly IConfigurationSection _config;
    private readonly DhcpServer _dhcp;
    private readonly WebUIProcessor _webUi;
    private PathString _basePath;

    private JsonSerializerSettings _serializeSettings;

    public WebApiProcessor(IConfigurationSection config, DhcpServer dhcp, WebUIProcessor webUi) : base(new Logger("HTTP.WebApi"))
    {
        _config = config;
        _dhcp = dhcp;
        _webUi = webUi;
    }

    protected override void OnInit()
    {
        _basePath = _config["Base"].TrimEnd('/');
        _serializeSettings = new JsonSerializerSettings { Converters = { IpAddressStringJsonConverter.Instance } };
    }

    public async Task<bool> ProcessRequest(HttpContext context)
    {
        if (!context.Request.Path.StartsWithSegments(_basePath, out var remaining)) return false;

        //TODO: AUTH?

        if (remaining.StartsWithSegments("/dhcp"))
        {
            //TODO: BLAZOR: Show vendor by mac address
            var json = JsonConvert.SerializeObject(_dhcp?.Pool?.Slots, _serializeSettings);
            context.Response.ContentType = "application/json";
            await context.Response.WriteAsync(json);
            return true;
        }

        if (remaining.StartsWithSegments("/reload-ui"))
        {
            _webUi.LoadContent();
            return true;
        }

        if (remaining.StartsWithSegments("/log", out remaining))
        {
            if (remaining.StartsWithSegments("/query", out var query))
            {
                //TODO: Log Query Filter/Paging/Search
                return true;
            }

            if (remaining.StartsWithSegments("/notify"))
            {
                //TODO:  Notify (WebSocket?)
                //LogPool.LogChanged+= ...
                return true;
            }

            if (remaining.StartsWithSegments("/clear"))
            {
                LogPool.Clear();
                Logger.LogInfo("Clear logs");
                return true;
            }

            var items = LogPool.TakeLast(1024);
            var json = JsonConvert.SerializeObject(items, _serializeSettings);
            context.Response.ContentType = "application/json";
            await context.Response.WriteAsync(json);
            return true;
        }
        return false;
    }
}
