﻿using NetBootServerCore.Components.Logging;
using Spectre.Console;

namespace NetBootServerCore.Components.LivingConsole
{
    public static class LiveConsole
    {
        private static readonly AutoResetEvent SleepEvent = new(false);

        private static bool _isRunning;
        private static Task _task;

        public static void Start()
        {
            _isRunning = true;
            LogPool.LogChanged += Refresh;
            _task = Task.Run(RunInternal);
        }

        public static void Stop()
        {
            LogPool.LogChanged -= Refresh;
            Refresh();
            _isRunning = false;
            _task.Wait(10000);
        }

        public static void Refresh() => SleepEvent.Set();

        private static void RunInternal()
        {
            Table tableMain;
            TableColumn ct1UpTime;
            TableColumn ct2RequestHandled;
            {
                tableMain = new Table { Border = TableBorder.None, Expand = true };
                var colMain = new TableColumn("");
                tableMain.AddColumn(colMain);
                var tableTitle = new Table();
                {
                    ct1UpTime = new TableColumn("");
                    ct2RequestHandled = new TableColumn("");
                    tableTitle.AddColumn(ct1UpTime);
                    tableTitle.AddColumn(ct2RequestHandled);
                }
                colMain.Header = tableTitle;

                tableMain.AddRow(new Panel("Logs Brrr"));
            }

            do
            {
                Console.Clear();
                var live = AnsiConsole.Live(tableMain);
                live.Cropping(VerticalOverflowCropping.Bottom)
                    .Start(ctx =>
                    {
                        var conW = Console.WindowWidth;
                        var conH = Console.WindowHeight;

                        void RefreshInternal()
                        {
                            var now = DateTime.Now;
                            var up = now - NbsProgram.LaunchTime;
                            ct2RequestHandled.Header = new Text($"Reqs: {LogPool.RequestsHandled} In: {LogPool.InBytes} Out: {LogPool.OutBytes}");
                            ct1UpTime.Header = new Text($"UP: {(int)up.TotalDays:000}d{up.Hours:00}h{up.Minutes:00}m{up.Seconds:00}s{up.Milliseconds:000}");
                            if (LogPool.Count != 0 && Console.WindowHeight > 4) tableMain.UpdateCell(0, 0, new Rows(LogPool.TakeLast(conH - 4).Select(p => LogMarkup.Create(p, conW))));
                            ctx.Refresh();
                        }

                        do
                        {
                            RefreshInternal();
                            SleepEvent.WaitOne(1000);
                        } while (_isRunning && conW == Console.WindowWidth && conH == Console.WindowHeight);

                        live.AutoClear = _isRunning;
                        if (_isRunning == false) RefreshInternal();
                    });
            } while (_isRunning);
        }
    }
}
