﻿using NetBootServerCore.Components.Logging;
using Spectre.Console;
using static Spectre.Console.Markup;
using LogLevel = NetBootServerCore.Components.Logging.LogLevel;

namespace NetBootServerCore.Components.LivingConsole
{
    public class LogMarkup
    {
        public static Markup Create(LogEntry log, int consoleWidth)
        {
            var color = "aqua";
            var level = "UNKNOWN";

            switch (log.Level)
            {
                case LogLevel.Debug:
                    color = "grey";
                    level = "DEBG";
                    break;

                case LogLevel.Verbose:
                    color = "navy";
                    level = "VERB";
                    break;

                case LogLevel.Info:
                    color = "lime";
                    level = "INFO";
                    break;

                case LogLevel.Warn:
                    color = "yellow";
                    level = "WARN";
                    break;

                case LogLevel.Error:
                    color = "red";
                    level = "ERRO";
                    break;

                case LogLevel.Fatal:
                    color = "fuchsia";
                    level = "FATL";
                    break;
            }

            var ts = $"{log.Time:dd-HH:mm:ss.fff} ";
            var len = ts.Length;

            var lt = $"[bold {color}]{Escape(level)}[/] {Escape($"[{log.Tag}]")} ";
            len += level.Length + 1 + (log.Tag?.Length ?? 0) + 3;

            var msg = $"{log.Message} {log.Details?.Split(Environment.NewLine, StringSplitOptions.RemoveEmptyEntries).FirstOrDefault()}";
            len += msg.Length;

            if (consoleWidth < len + 3)
            {
                var sub = msg.Length - (len + 3 - consoleWidth);

                msg = sub > 0
                    ? msg.Substring(0, sub)
                    : "";
            }

            return new Markup($" {Escape(ts)}{lt}{Escape(msg)}");
        }
    }
}
