﻿namespace NetBootServerCore.Components.Logging
{
    public static class LogPool
    {
        //TODO: Data Logs

        public static event Action LogChanged;

        public static int RequestsHandled = 0;
        public static int InBytes = 0;
        public static int OutBytes = 0;

        private static readonly List<LogEntry> Logs = new();

        public static void Append(LogEntry entry)
        {
            lock (Logs)
            {
                Logs.Add(entry);
                LogChanged?.Invoke();
            }
        }

        public static void Clear()
        {
            lock (Logs)
            {
                Logs.Clear();
                LogChanged?.Invoke();
            }
        }

        public static LogEntry[] TakeLast(int num)
        {
            lock (Logs) return Logs.TakeLast(num).ToArray();
        }

        public static LogEntry[] Paging(int skip, int take)
        {
            lock (Logs) return Logs.Skip(skip).Take(take).ToArray();
        }

        public static int Count
        {
            get
            {
                lock (Logs) return Logs.Count;
            }
        }

        public static LogEntry GetEntryById(int id)
        {
            lock (Logs) return Logs.FirstOrDefault(p => p.Id == id);
        }
    }
}
