﻿using NetBootServerCore.Components.Logging;
using System.Net;
using Tftp.Net;

namespace NetBootServerCore.Components.NetBoot
{
    public class NbsTftpServer : NbsComponentBase
    {
        private const int Port = 69;

        private readonly IConfigurationSection _config;

        public NbsTftpServer(IConfigurationSection config) : base(new Logger("TFTP")) => _config = config;

        // import Callisto82/tftp.net

        private IPAddress _listenAddress;
        private string _rootDir;

        private TftpServer _server;
        private bool _enable;

        private void ProcessOnReadRequest(ITftpTransfer transfer, EndPoint client)
        {
            var path = Path.Combine(_rootDir, transfer.Filename);
            var file = new FileInfo(path);

            if (!file.FullName.StartsWith(_rootDir, StringComparison.InvariantCultureIgnoreCase))
            {
                Logger.LogWarn($"Invalid access {file}");
                transfer.Cancel(TftpErrorPacket.AccessViolation);
            }
            else if (!file.Exists)
            {
                Logger.LogWarn($"File not found {file}");
                transfer.Cancel(TftpErrorPacket.FileNotFound);
            }
            else
            {
                Logger.LogVerbose($"Request {file}");
                transfer.Start(new FileStream(file.FullName, FileMode.Open, FileAccess.Read));
            }
        }

        protected override void OnInit()
        {
            _enable = bool.Parse(_config["Enable"]);
            if (!_enable)
            {
                Logger.LogInfo("Disabled by config");
                return;
            }

            _server?.Dispose();

            _listenAddress = IPAddress.Parse(_config["ListenOn"]);
            _rootDir = Path.GetFullPath(_config["RootDir"]);

            _server = new TftpServer(_listenAddress, Port);
            _server.OnReadRequest += ProcessOnReadRequest;
        }

        protected override void OnStart()
        {
            if (!_enable)
            {
                Logger.LogInfo("Disabled by config");
                return;
            }

            _server.Start();
        }

        protected override void OnStop()
        {
            if (!_enable)
            {
                Logger.LogInfo("Disabled by config");
                return;
            }

            _server?.Dispose();
            _server = null;
        }
    }
}
